@@ -12,7 +12,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized redefine );
 
-our $VERSION = '1.006';
+our $VERSION = '1.007';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
@@ -78,7 +78,7 @@ sub import {
       $_THAW   = shift, next if ( $_arg eq 'thaw' );
 
       if ( $_arg eq 'sereal' ) {
-         if ( shift eq '1' ) {
+         if ( shift eq '1' && !exists $INC{'PDL.pm'} ) {
             local $@; eval 'use Sereal qw( encode_sereal decode_sereal )';
             if ( !$@ ) {
                $_FREEZE = sub { encode_sereal( @_, { freeze_callbacks => 1 } ) };
@@ -132,16 +132,17 @@ sub create {
 
    ## error checking -- --- --- --- --- --- --- --- --- --- --- --- --- --- ---
 
-   if ( $^O eq 'MSWin32' && $INC{'MCE.pm'} && MCE->wid() ) {
-      return $self->_error(
-         "spawning MCE::Hobo by MCE worker is not supported on MSWin32\n"
-      );
-   }
-
-   if ( $INC{'MCE.pm'} && $MCE::MCE->{use_threads} && MCE->wid() ) {
-      return $self->_error(
-         "spawning MCE::Hobo by threaded MCE worker is not supported\n"
-      );
+   if ( $INC{'MCE.pm'} && MCE->wid() ) {
+      if ( $^O eq 'MSWin32' ) {
+         return $self->_error(
+            "spawning MCE Hobo by MCE worker is not supported on MSWin32\n"
+         );
+      }
+      if ( $MCE::MCE->{use_threads} ) {
+         return $self->_error(
+            "spawning MCE Hobo by threaded MCE worker is not supported\n"
+         );
+      }
    }
 
    if ( ref($func) ne 'CODE' && !length($func) ) {
@@ -191,7 +192,7 @@ sub create {
       local $| = 1;
 
       $SIG{QUIT} = \&_exit; $SIG{TERM} = $SIG{INT} = $SIG{HUP} = \&_trap;
-      MCE::Shared::init() if ( !$INC{'MCE.pm'} || !MCE->wid() );
+      MCE::Shared::init($_id);
 
       $_SELF = $self, $_SELF->{WRK_ID} = $wrk_id, $_SELF->{PID} = $$;
       $_LIST = undef;
@@ -503,8 +504,7 @@ sub _exit {
 }
 
 sub _trap {
-   local $\;
-   $SIG{ $_[0] } = sub { };
+   local $\; $SIG{ $_[0] } = sub { };
    print {*STDERR} "Signal $_[0] received in process $$.$_tid\n";
 
    _exit();
@@ -526,7 +526,7 @@ MCE::Hobo - A threads-like parallelization module
 
 =head1 VERSION
 
-This document describes MCE::Hobo version 1.006
+This document describes MCE::Hobo version 1.007
 
 =head1 SYNOPSIS
 
