@@ -12,7 +12,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized numeric once );
 
-our $VERSION = '1.006';
+our $VERSION = '1.007';
 
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
@@ -49,6 +49,7 @@ BEGIN {
 
    eval 'use IO::FDPass' if !$INC{'IO/FDPass.pm'} && $^O ne 'cygwin';
    eval 'PDL::no_clone_skip_warning()' if $INC{'PDL.pm'};
+   eval 'use PDL::IO::Storable' if $INC{'PDL.pm'};
 }
 
 use MCE::Util ();
@@ -106,6 +107,14 @@ my $LF = "\012"; Internals::SvREADONLY($LF, 1);
 
 my $_is_MSWin32 = ($^O eq 'MSWin32') ? 1 : 0;
 my $_tid = $_has_threads ? threads->tid() : 0;
+my $_oid = "$$.$_tid";
+
+my %_iter_allow = (qw/
+   MCE::Shared::Array   1
+   MCE::Shared::Hash    1
+   MCE::Shared::Ordhash 1
+   Hash::Ordered        1
+/);
 
 sub _croak { goto &Carp::croak }
 sub  CLONE { $_tid = threads->tid() if $_has_threads }
@@ -199,13 +208,16 @@ sub _new {
 
    my ($_buf, $_id, $_len);
 
-   my $_chn        = 1;
+   my $_chn = ($_has_threads)
+      ? $_tid % $_SVR->{_data_channels} + 1
+      : abs($$) % $_SVR->{_data_channels} + 1;
+
    my $_DAT_LOCK   = $_SVR->{'_mutex_'.$_chn};
    my $_DAT_W_SOCK = $_SVR->{_dat_w_sock}->[0];
    my $_DAU_W_SOCK = $_SVR->{_dat_w_sock}->[$_chn];
 
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    $_DAT_LOCK->lock();
    print {$_DAT_W_SOCK} SHR_M_NEW.$LF . $_chn.$LF;
@@ -247,13 +259,16 @@ sub _new {
 sub _incr_count {
    return unless $_svr_pid;
 
-   my $_chn        = 1;
+   my $_chn = ($_has_threads)
+      ? $_tid % $_SVR->{_data_channels} + 1
+      : abs($$) % $_SVR->{_data_channels} + 1;
+
    my $_DAT_LOCK   = $_SVR->{'_mutex_'.$_chn};
    my $_DAT_W_SOCK = $_SVR->{_dat_w_sock}->[0];
    my $_DAU_W_SOCK = $_SVR->{_dat_w_sock}->[$_chn];
 
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    $_DAT_LOCK->lock();
    print {$_DAT_W_SOCK} SHR_M_INC.$LF . $_chn.$LF;
@@ -289,10 +304,6 @@ sub _share {
          else                         { pdl      (@{ $_item }) }
       };
    }
-   elsif (!exists $INC{ join('/',split(/::/,$_class)).'.pm' }) {
-      local $@; local $SIG{__DIE__};
-      eval "use $_class ()";
-   }
 
    $_all{ $_id } = $_class; $_ob3{ "$_id:count" } = 1;
 
@@ -317,11 +328,12 @@ sub _start {
    $SIG{HUP} = $SIG{INT} = $SIG{PIPE} = $SIG{QUIT} = $SIG{TERM} = \&_trap
       if (!$_is_MSWin32 && !$INC{'MCE/Signal.pm'});
 
-   my $_data_channels = ($INC{'MCE.pm'} && MCE->wid()) ? 1 : DATA_CHANNELS;
-   my $_is_child = $_has_threads ? 0 : 1;
+   local $_; $_init_pid = "$$.$_tid";
+
+   my $_data_channels = ($_oid eq $_init_pid) ? DATA_CHANNELS : 4;
+   my $_is_child = ($_has_threads && $^O eq 'MSWin32') ? 0 : 1;
 
    $_SVR = { _data_channels => $_data_channels };
-   local $_; $_init_pid = "$$.$_tid";
 
    MCE::Util::_sock_pair($_SVR, qw(_dat_r_sock _dat_w_sock), $_)
       for (0 .. $_data_channels);
@@ -504,10 +516,7 @@ sub _loop {
       if (!exists $_itr{ $_id }) {
 
          # MCE::Shared::{ Array, Hash, Ordhash }, Hash::Ordered
-         if (
-            $_all{ $_id } =~ /^MCE::Shared::(?:Array|Hash|Ordhash)$/ ||
-            $_all{ $_id } eq 'Hash::Ordered'
-         ) {
+         if ($_iter_allow{ $_all{ $_id } } && $_obj{ $_id }->can('keys')) {
             my @_keys = ( exists $_itr{ "$_id:args" } )
                ? $_obj{ $_id }->keys( @{ $_itr{ "$_id:args" } } )
                : $_obj{ $_id }->keys;
@@ -523,55 +532,6 @@ sub _loop {
             };
          }
 
-         # MCE::Shared::{ Minidb }
-         elsif ( $_all{ $_id } eq 'MCE::Shared::Minidb' ) {
-            @{ $_itr{ "$_id:args" } } = () unless exists($_itr{ "$_id:args" });
-
-            my @_a = @{ $_itr{ "$_id:args" } };
-            my $_data; my $_ta = 0;
-
-            if ( $_a[0] =~ /^:lists$/i ) {
-               $_data = $_obj{ $_id }->[1][0];
-               shift @_a;
-               if ( ! @_a ) {
-                  @_a = $_obj{ $_id }->lkeys();
-               }
-               elsif ( @_a == 1 && $_a[0] =~ /^(?:key|\S+)[ ]+\S\S?[ ]+\S/ ) {
-                  @_a = $_obj{ $_id }->lkeys(@_a);
-               }
-               elsif ( @_a == 2 && $_a[1] =~ /^(?:key|val)[ ]+\S\S?[ ]+\S/ ) {
-                  $_data = $_obj{ $_id }->[1][0]->{ $_a[0] };
-                  $_ta = 1, @_a = $_obj{ $_id }->lkeys(@_a);
-               }
-            }
-            else {
-               $_data = $_obj{ $_id }->[0][0];
-               shift @_a if ( $_a[0] =~ /^:hashes$/i );
-               if ( ! @_a ) {
-                  @_a = $_obj{ $_id }->hkeys();
-               }
-               elsif ( @_a == 1 && $_a[0] =~ /^(?:key|\S+)[ ]+\S\S?[ ]+\S/ ) {
-                  @_a = $_obj{ $_id }->hkeys(@_a);
-               }
-               elsif ( @_a == 2 && $_a[1] =~ /^(?:key|val)[ ]+\S\S?[ ]+\S/ ) {
-                  $_data = $_obj{ $_id }->[0][0]->{ $_a[0] };
-                  @_a = $_obj{ $_id }->hkeys(@_a);
-               }
-            }
-
-            $_itr{ $_id } = sub {
-               my $_key = shift @_a;
-               if ( !defined $_key ) {
-                  print {$_DAU_R_SOCK} '-1'.$LF;
-                  return;
-               }
-               my $_buf = $_freeze->([
-                  $_key, $_ta ? $_data->[ $_key ] : $_data->{ $_key }
-               ]);
-               print {$_DAU_R_SOCK} length($_buf).$LF, $_buf;
-            };
-         }
-
          # Not supported
          else {
             print {$_DAU_R_SOCK} '-1'.$LF;
@@ -589,11 +549,18 @@ sub _loop {
    my %_output_function = (
 
       SHR_M_NEW.$LF => sub {                      # New share
-         my ($_buf, $_params, $_args, $_fd, $_item, %_hndls);
+         my ($_buf, $_params, $_class, $_args, $_fd, $_item, %_hndls);
 
          chomp($_len = <$_DAU_R_SOCK>);
          read $_DAU_R_SOCK, $_buf, $_len;
+
          $_params = $_thaw->($_buf);
+         $_class  = $_params->{'class'};
+
+         if (!exists $INC{ join('/',split(/::/,$_class)).'.pm' }) {
+            local $@; local $SIG{__DIE__};
+            eval "use $_class ()";
+         }
 
          chomp($_len = <$_DAU_R_SOCK>);
          read $_DAU_R_SOCK, $_buf, $_len;
@@ -628,7 +595,7 @@ sub _loop {
 
       SHR_M_CID.$LF => sub {                      # ClientID request
          print {$_DAU_R_SOCK} (++$_client_id).$LF;
-         $_client_id = 0 if ($_client_id > 2e6);
+         $_client_id = 0 if ($_client_id > 2e9);
 
          return;
       },
@@ -851,10 +818,7 @@ sub _loop {
             my $_buf;
 
             # MCE::Shared::{ Array, Hash, Ordhash }, Hash::Ordered
-            if (
-               $_all{ $_id } =~ /^MCE::Shared::(?:Array|Hash|Ordhash)$/ ||
-               $_all{ $_id } eq 'Hash::Ordered'
-            ) {
+            if ($_iter_allow{ $_all{ $_id } } && $_obj{ $_id }->can('clone')) {
                $_buf = ($_len)
                   ? $_freeze->($_obj{ $_id }->clone(@{ $_thaw->($_keys) }))
                   : $_freeze->($_obj{ $_id });
@@ -1000,6 +964,7 @@ sub _loop {
          chomp($_id = <$_DAU_R_SOCK>);
 
          close $_obj{ $_id } if defined fileno($_obj{ $_id });
+         print {$_DAU_R_SOCK} $LF;
 
          return;
       },
@@ -1025,33 +990,72 @@ sub _loop {
 
          if (@{ $_args } == 2) {
             open( $_obj{ $_id }, "$_args->[0]", $_args->[1] )
-               or do { $_err = Scalar::Util::dualvar($!,$!) };
-         }
-         else {
+               or do { $_err = 0+$! };
+         } else {
             open( $_obj{ $_id }, $_args->[0] )
-               or do { $_err = Scalar::Util::dualvar($!,$!) };
+               or do { $_err = 0+$! };
          }
 
-         if ($_len = length($_err)) {
-            print {$_DAU_R_SOCK} $_len.$LF . (0+$_err).$LF . ''.$_err;
-         }
-         else {
-            print {$_DAU_R_SOCK} '0'.$LF . $LF;
-         }
+         print {$_DAU_R_SOCK} $_err.$LF;
 
          return;
       },
 
       SHR_O_REA.$LF => sub {                      # Handle READ
-         my ($_buf, $_a3); local $!;
+         my ($_a3, $_auto);
 
-         chomp($_id = <$_DAU_R_SOCK>),
-         chomp($_a3 = <$_DAU_R_SOCK>);
+         chomp($_id  = <$_DAU_R_SOCK>),
+         chomp($_a3  = <$_DAU_R_SOCK>),
+         chomp($_len = <$_DAU_R_SOCK>);
 
-         my $_fh = $_obj{ $_id };
-         read($_fh, $_buf, $_a3);
+         if (lc(substr $_a3, -1, 1) eq 'm') {
+            $_auto = 1, chop $_a3; $_a3 *= 1024 * 1024;
+         } elsif (lc(substr $_a3, -1, 1) eq 'k') {
+            $_auto = 1, chop $_a3; $_a3 *= 1024;
+         }
 
-         print {$_DAU_R_SOCK} length($_buf).$LF, $_buf;
+         local $/; read($_DAU_R_SOCK, $/, $_len) if ($_len);
+         my ($_fh, $_buf) = ($_obj{ $_id }); local ($!, $.);
+
+         # support special case; e.g. $/ = "\n>" for bioinformatics
+         # anchoring ">" at the start of line
+
+         if (!$_auto) {
+            $. = 0, $_ret = read($_fh, $_buf, $_a3);
+         }
+         elsif (!eof($_fh)) {
+            if (length $/ > 1 && substr($/, 0, 1) eq "\n") {
+               $_len = length($/) - 1;
+
+               if (tell $_fh) {
+                  $_buf = substr($/, 1);
+                  $_ret = read($_fh, $_buf, $_a3, length($_buf));
+               } else {
+                  $_ret = read($_fh, $_buf, $_a3);
+               }
+
+               if (defined $_ret) {
+                  $.    += 1 if eof($_fh);
+                  $_buf .= readline($_fh);
+
+                  substr($_buf, -$_len, $_len, '')
+                     if (substr($_buf, -$_len) eq substr($/, 1));
+               }
+            }
+            elsif (defined ($_ret = read($_fh, $_buf, $_a3))) {
+               $.    += 1 if eof($_fh);
+               $_buf .= readline($_fh);
+            }
+         }
+         else {
+            $_buf = '', $_ret = 0;
+         }
+
+         if (defined $_ret) {
+            print {$_DAU_R_SOCK} "$.$LF" . length($_buf).$LF, $_buf;
+         } else {
+            print {$_DAU_R_SOCK} "$.$LF" . ( (0+$!) * -1 ).$LF;
+         }
 
          return;
       },
@@ -1061,30 +1065,32 @@ sub _loop {
          chomp($_len = <$_DAU_R_SOCK>);
 
          local $/; read($_DAU_R_SOCK, $/, $_len) if ($_len);
-         my ($_fh, $_buf) = ($_obj{ $_id }); local $!;
+         my ($_fh, $_buf) = ($_obj{ $_id }); local ($!, $.);
 
          # support special case; e.g. $/ = "\n>" for bioinformatics
          # anchoring ">" at the start of line
 
-         if (!eof($_fh)) {
-            if (length $/ > 1 && substr($/, 0, 1) eq "\n") {
-               $_len = length($/) - 1;
+         if (length $/ > 1 && substr($/, 0, 1) eq "\n" && !eof($_fh)) {
+            $_len = length($/) - 1;
 
-               if (tell $_fh) {
-                  $_buf = substr($/, 1), $_buf .= readline($_fh);
-               } else {
-                  $_buf = readline($_fh);
-               }
-
-               substr($_buf, -$_len, $_len, '')
-                  if (substr($_buf, -$_len) eq substr($/, 1));
-            }
-            else {
+            if (tell $_fh) {
+               $_buf = substr($/, 1), $_buf .= readline($_fh);
+            } else {
                $_buf = readline($_fh);
             }
+
+            substr($_buf, -$_len, $_len, '')
+               if (substr($_buf, -$_len) eq substr($/, 1));
+         }
+         else {
+            $_buf = readline($_fh);
          }
 
-         print {$_DAU_R_SOCK} "$.$LF" . length($_buf).$LF, $_buf;
+         if (defined $_buf) {
+            print {$_DAU_R_SOCK} "$.$LF" . length($_buf).$LF, $_buf;
+         } else {
+            print {$_DAU_R_SOCK} "$.$LF" . ( (0+$!) * -1 ).$LF;
+         }
 
          return;
       },
@@ -1244,8 +1250,18 @@ sub _loop {
 
          if ($_all{ $_id } eq 'PDL') {
             local @_ = @{ $_thaw->($_buf) };
-            $_obj{ $_id }->slice( $_[0] ) .= $_[1]  if @_ == 2;
-            ins( inplace( $_obj{ $_id } ), @_ )     if @_  > 2;
+            if (@_ == 1) {
+               # ins( inplace( $this ), $what, 0, 0 );
+               ins( inplace( $_obj{ $_id } ), @_, 0, 0 );
+            }
+            elsif (@_ == 2) {
+               # $this->slice( $arg1 ) .= $arg2;
+               $_obj{ $_id }->slice( $_[0] ) .= $_[1];
+            }
+            elsif (@_ > 2) {
+               # ins( inplace( $this ), $what, @coords );
+               ins( inplace( $_obj{ $_id } ), @_ );
+            }
          }
 
          return;
@@ -1292,8 +1308,6 @@ sub _loop {
             $_warn2->($_fn, blessed($_obj{ $_id }));
          }
 
-         print {$_DAU_R_SOCK} $LF;
-
          return;
       },
 
@@ -1309,27 +1323,39 @@ sub _loop {
 
       my $_val_bytes = "\x00\x00\x00\x00";
       my $_ptr_bytes = unpack('I', pack('P', $_val_bytes));
-      my $_nbytes; my $_count = 0;
+      my $_nbytes; my $_count = 1; my $_retries = 0;
       my $_done = 0;
 
       while (!$_done) {
-         ioctl($_DAT_R_SOCK, 0x4004667f, $_ptr_bytes);  # MSWin32 FIONREAD
 
-         unless ($_nbytes = unpack('I', $_val_bytes)) {
-            # delay so not to consume a CPU for non-blocking ioctl
-            $_count = 0, sleep 0.015 if ++$_count > 1618;
+         RETRY: {
+            ioctl($_DAT_R_SOCK, 0x4004667f, $_ptr_bytes);  # MSWin32 FIONREAD
+
+            unless ($_nbytes = unpack('I', $_val_bytes)) {
+               # delay so not to consume a CPU from non-blocking ioctl
+               if ($_count) {
+                  if (++$_count > 1280) {
+                     $_count = 1, sleep 0.015;
+                     $_count = 0 if ++$_retries == 3;
+                  }
+               }
+               else {
+                  sleep 0.045;
+               }
+               redo RETRY;
+            }
          }
-         else {
-            $_count = 0;
-            do {
-               sysread($_DAT_R_SOCK, $_func, 8);
-               $_done = 1, last() unless length($_func) == 8;
 
-               $_DAU_R_SOCK = $_channels->[ substr($_func, -2, 2, '') ];
-               $_output_function{$_func}();
+         $_count = 1, $_retries = 0;
 
-            } while (($_nbytes -= 8) >= 8);
-         }
+         do {
+            sysread($_DAT_R_SOCK, $_func, 8);
+            $_done = 1, last() unless length($_func) == 8;
+
+            $_DAU_R_SOCK = $_channels->[ substr($_func, -2, 2, '') ];
+            $_output_function{$_func}();
+
+         } while (($_nbytes -= 8) >= 8);
       }
    }
    else {
@@ -1344,6 +1370,10 @@ sub _loop {
 
    # Exiting via POSIX's _exit to avoid END blocks.
 
+   for ( values %_obj ) {
+      close $_ if ( ref($_) eq 'MCE::Shared::Handle' && defined fileno($_) );
+   }
+
    $_is_child ? POSIX::_exit(0) : threads->exit(0);
 
    return;
@@ -1373,10 +1403,10 @@ use constant {
    _CLASS => 1, _DREF  => 2, _ITER   => 3,  # shared object
 };
 
-my %_hash_support = (qw/
-   MCE::Shared::Hash     1
-   MCE::Shared::Ordhash  1
-   Hash::Ordered         1
+my %_hash_deref_allow = (qw/
+   MCE::Shared::Hash    1
+   MCE::Shared::Ordhash 1
+   Hash::Ordered        1
 /);
 
 use overload (
@@ -1392,7 +1422,7 @@ use overload (
    },
    q(%{})   => sub {
       $_[0]->[_DREF] || do {
-         return $_[0] if !exists $_hash_support{ $_[0]->[_CLASS] };
+         return $_[0] unless $_hash_deref_allow{ $_[0]->[_CLASS] };
          tie my %h, __PACKAGE__, bless([ $_[0]->[0] ], __PACKAGE__);
          $_[0]->[_DREF] = \%h;
       };
@@ -1417,8 +1447,8 @@ my $_ready   = \&MCE::Util::_sock_ready;
 # Hook for threads.
 
 sub CLONE {
-   %_new = ();
-   &_init(threads->tid()) if ($_has_threads && !$INC{'MCE.pm'});
+   $_tid = threads->tid() if $_has_threads;
+   %_new = (), &_init($_tid) if (!$INC{'MCE.pm'} || MCE->wid == 0);
 }
 
 # Private functions.
@@ -1468,7 +1498,7 @@ sub _get_client_id {
    my $_ret;
 
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    $_dat_ex->();
    print {$_DAT_W_SOCK} 'M~CID'.$LF . $_chn.$LF;
@@ -1481,7 +1511,7 @@ sub _get_client_id {
 sub _init {
    return unless defined $_SVR;
 
-   my $_wid = $_[0] || &_get_client_id();
+   my $_wid = $_[0] // &_get_client_id();
       $_wid = $$ if ( $_wid !~ /\d+/ );
 
    $_chn        = abs($_wid) % $_SVR->{_data_channels} + 1;
@@ -1544,7 +1574,7 @@ sub _auto {
    }
 
    if ( $_wa ) {
-      local $/ = $LF if (!$/ || $/ ne $LF);
+      local $/ = $LF if ($/ ne $LF);
       chomp($_len = <$_DAU_W_SOCK>);
 
       my $_frozen = chop($_len);
@@ -1561,11 +1591,11 @@ sub _auto {
    }
 }
 
-# Called by broadcast, signal, timedwait, and rewind.
+# Called by CLOSE, broadcast, signal, timedwait, and rewind.
 
 sub _req1 {
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    $_dat_ex->();
    print {$_DAT_W_SOCK} $_[0].$LF . $_chn.$LF;
@@ -1577,7 +1607,7 @@ sub _req1 {
    $_ret;
 }
 
-# Called by DESTROY, STORE, CLOSE, PRINT, PRINTF, timedwait, wait, await,
+# Called by DESTROY, STORE, PRINT, PRINTF, timedwait, wait, await,
 # ins_inplace, and destroy.
 
 sub _req2 {
@@ -1595,7 +1625,7 @@ sub _req2 {
 
 sub _req3 {
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    $_dat_ex->();
    print {$_DAT_W_SOCK} $_[0].$LF . $_chn.$LF;
@@ -1617,7 +1647,7 @@ sub _req3 {
 
 sub _req4 {
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    $_dat_ex->();
    print {$_DAT_W_SOCK} $_[0].$LF . $_chn.$LF;
@@ -1643,7 +1673,7 @@ sub _req4 {
 
 sub _req5 {
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    $_dat_ex->();
    print {$_DAT_W_SOCK} 'M~SZE'.$LF . $_chn.$LF;
@@ -1659,7 +1689,7 @@ sub _req5 {
 
 sub _req6 {
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    $_dat_ex->();
    print {$_DAT_W_SOCK} 'O~FCH'.$LF . $_chn.$LF;
@@ -1681,15 +1711,13 @@ sub _req6 {
 sub _req7 {
    my ( $_fn, $self ) = @_;
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    delete $self->[_ITER] if defined $self->[_ITER];
 
    $_dat_ex->();
    print {$_DAT_W_SOCK} 'O~CLR'.$LF . $_chn.$LF;
    print {$_DAU_W_SOCK} $self->[0].$LF . $_fn.$LF;
-
-   <$_DAU_W_SOCK>;
    $_dat_un->();
 
    return;
@@ -1741,7 +1769,8 @@ sub destroy {
 # export ( )
 
 sub export {
-   my $_id   = shift()->[0];
+   my $_ob   = shift;
+   my $_id   = $_ob->[0];
    my $_lkup = ref($_[0]) eq 'HASH' ? shift : {};
 
    # safety for circular references to not loop endlessly
@@ -1749,62 +1778,50 @@ sub export {
 
    my $_tmp   = @_ ? $_freeze->([ @_ ]) : '';
    my $_buf   = $_id.$LF . length($_tmp).$LF;
-   my $_item  = $_lkup->{ $_id } = _req3('M~EXP', $_buf, $_tmp);
-   my $_class = $_blessed->($_item);
+   my $_class = $_ob->[_CLASS];
 
-   # MCE::Shared::{ Array, Hash, Ordhash }, Hash::Ordered
-   if (
-      $_class =~ /^MCE::Shared::(?:Array|Hash|Ordhash)$/ ||
-      $_class eq 'Hash::Ordered'
-   ) {
-      require MCE::Shared::Array   if $_class eq 'MCE::Shared::Array';
-      require MCE::Shared::Hash    if $_class eq 'MCE::Shared::Hash';
-      require MCE::Shared::Ordhash if $_class eq 'MCE::Shared::Ordhash';
-      require Hash::Ordered        if $_class eq 'Hash::Ordered';
-
-      for my $k ($_item->keys) {
-         if ($_blessed->($_item->get($k)) && $_item->get($k)->can('export')) {
-            $_item->set($k, $_item->get($k)->export($_lkup));
-         }
-      }
+   if (!exists $INC{ join('/',split(/::/,$_class)).'.pm' }) {
+      local $@; local $SIG{__DIE__};
+      eval "use $_class ()";
    }
 
-   # MCE::Shared::{ Scalar }
-   elsif ( $_class eq 'MCE::Shared::Scalar' ) {
-      require MCE::Shared::Scalar;
+   my $_item = $_lkup->{ $_id } = _req3('M~EXP', $_buf, $_tmp);
+   my $_data;
 
+   if ( $_class eq 'MCE::Shared::Array' ) {
+      ## no critic
+      map { $_ = $_->export($_lkup) if $_blessed->($_) && $_->can('export') }
+         @{ $_item };
+   }
+   elsif ( $_class eq 'MCE::Shared::Scalar' ) {
       if ($_blessed->($_item->get()) && $_item->get()->can('export')) {
          $_item->set($_item->get()->export($_lkup));
       }
    }
+   else {
+      if    ( $_class eq 'MCE::Shared::Hash'    ) { $_data = $_item      }
+      elsif ( $_class eq 'MCE::Shared::Ordhash' ) { $_data = $_item->[0] }
+      elsif ( $_class eq 'Hash::Ordered'        ) { $_data = $_item->[0] }
+
+      ## no critic
+      map { $_ = $_->export($_lkup) if $_blessed->($_) && $_->can('export') }
+         values %{ $_data } if defined( $_data );
+   }
 
    $_item;
 }
 
-# iterator ( ":hashes", key, "query string" )  # Minidb HoH
-# iterator ( ":hashes", key [, key, ... ] )
-# iterator ( ":hashes", "query string" )
-# iterator ( ":hashes" )
-#
-# iterator ( ":lists", key, "query string" )   # Minidb HoA
-# iterator ( ":lists", key [, key, ... ] )
-# iterator ( ":lists", "query string" )
-# iterator ( ":lists" )
-#
-# iterator ( index, [, index, ... ] )          # Array/Hash/Ordhash
+# iterator ( index, [, index, ... ] )
 # iterator ( key, [, key, ... ] )
 # iterator ( "query string" )
 # iterator ( )
 
 sub iterator {
    my ( $self, @keys ) = @_;
-   my $ref = $self->blessed();
+   my $pkg = $self->blessed();
 
    # MCE::Shared::{ Array, Hash, Ordhash }, Hash::Ordered
-   if (
-      $ref =~ /^MCE::Shared::(?:Array|Hash|Ordhash)$/ ||
-      $ref eq 'Hash::Ordered'
-   ) {
+   if ( $_iter_allow{ $pkg } && eval qq{ $pkg->can('keys') } ) {
       if ( ! @keys ) {
          @keys = $self->keys;
       }
@@ -1818,73 +1835,15 @@ sub iterator {
       };
    }
 
-   # MCE::Shared::{ Minidb }
-   elsif ( $ref eq 'MCE::Shared::Minidb' ) {
-      if ( $keys[0] =~ /^:lists$/i ) {
-         shift @keys;
-         if ( ! @keys ) {
-            @keys = $self->lkeys;
-         }
-         elsif ( @keys == 1 && $keys[0] =~ /^(?:key|\S+)[ ]+\S\S?[ ]+\S/ ) {
-            @keys = $self->lkeys(@keys);
-         }
-         elsif ( @keys == 2 && $keys[1] =~ /^(?:key|val)[ ]+\S\S?[ ]+\S/ ) {
-            my $key = $keys[0];  @keys = $self->lkeys(@keys);
-            return sub {
-               return unless @keys;
-               my $field = shift(@keys);
-               return ( $field => $self->lget($key, $field) );
-            };
-         }
-         return sub {
-            return unless @keys;
-            my $key = shift(@keys);
-            return ( $key => $self->lget($key) );
-         };
-      }
-      else {
-         shift @keys if ( $keys[0] =~ /^:hashes$/i );
-         if ( ! @keys ) {
-            @keys = $self->hkeys;
-         }
-         elsif ( @keys == 1 && $keys[0] =~ /^(?:key|\S+)[ ]+\S\S?[ ]+\S/ ) {
-            @keys = $self->hkeys(@keys);
-         }
-         elsif ( @keys == 2 && $keys[1] =~ /^(?:key|val)[ ]+\S\S?[ ]+\S/ ) {
-            my $key = $keys[0];  @keys = $self->hkeys(@keys);
-            return sub {
-               return unless @keys;
-               my $field = shift(@keys);
-               return ( $field => $self->hget($key, $field) );
-            };
-         }
-         return sub {
-            return unless @keys;
-            my $key = shift(@keys);
-            return ( $key => $self->hget($key) );
-         };
-      }
-   }
-
    # Not supported
    else {
       return sub { };
    }
 }
 
-# rewind ( begin, end, [ step, format ] )      # Sequence
+# rewind ( begin, end, [ step, format ] )
 #
-# rewind ( ":hashes", key, "query string" )    # Minidb HoH
-# rewind ( ":hashes", key [, key, ... ] )
-# rewind ( ":hashes", "query string" )
-# rewind ( ":hashes" )
-#
-# rewind ( ":lists", key, "query string" )     # Minidb HoA
-# rewind ( ":lists", key [, key, ... ] )
-# rewind ( ":lists", "query string" )
-# rewind ( ":lists" )
-#
-# rewind ( index [, index, ... ] )             # Array/(Ord)Hash
+# rewind ( index [, index, ... ] )
 # rewind ( key [, key, ... ] )
 # rewind ( "query string" )
 # rewind ( )
@@ -1901,7 +1860,7 @@ sub rewind {
 
 sub next {
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    $_dat_ex->();
    print {$_DAT_W_SOCK} 'M~INX'.$LF . $_chn.$LF;
@@ -2035,7 +1994,7 @@ sub wait {
 ###############################################################################
 
 sub CLOSE {
-   _req2('O~CLO', $_[0]->[0].$LF, '');
+   _req1('O~CLO', $_[0]->[0].$LF);
 }
 
 sub OPEN {
@@ -2064,7 +2023,7 @@ sub OPEN {
    }
 
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    $_dat_ex->();
    print {$_DAT_W_SOCK} 'O~OPN'.$LF . $_chn.$LF;
@@ -2072,64 +2031,69 @@ sub OPEN {
    <$_DAU_W_SOCK>;
 
    IO::FDPass::send( fileno $_DAU_W_SOCK, fileno $_fd ) if ($_fd > 2);
-
-   chomp(my $_len = <$_DAU_W_SOCK>);
    chomp(my $_err = <$_DAU_W_SOCK>);
-
-   read($_DAU_W_SOCK, my($_str), $_len) if $_len;
-
    $_dat_un->();
 
-   if ($_len) {
-      $! = Scalar::Util::dualvar($_err, $_str);
-      '';
-   }
-   else {
-      $! = Scalar::Util::dualvar(0, '');
-      1;
+   if ($_err) {
+      $! = $_err; '';
+   } else {
+      $! = 0; 1;
    }
 }
 
 sub READ {
-   my $_id  = $_[0]->[0];
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
 
    $_dat_ex->();
    print {$_DAT_W_SOCK} 'O~REA'.$LF . $_chn.$LF;
-   print {$_DAU_W_SOCK} $_id.$LF . $_[2].$LF;
+   print {$_DAU_W_SOCK} $_[0]->[0].$LF . $_[2].$LF . length($/).$LF . $/;
 
+   local $/ = $LF if ($/ ne $LF);
+   chomp(my $_ret = <$_DAU_W_SOCK>);
    chomp(my $_len = <$_DAU_W_SOCK>);
 
    if ($_len) {
+      if ($_len < 0) {
+         $_dat_un->(); $. = 0, $! = $_len * -1;
+         return undef;
+      }
       (defined $_[3])
          ? read($_DAU_W_SOCK, $_[1], $_len, $_[3])
          : read($_DAU_W_SOCK, $_[1], $_len);
    }
    else {
-      my $_ref = \$_[1]; $$_ref = '';
+      my $_ref = \$_[1];
+      if (defined $_[3]) {
+         substr($$_ref, $_[3], length($$_ref) - $_[3], '');
+      } else {
+         $$_ref = '';
+      }
    }
 
-   $_dat_un->();
+   $_dat_un->(); $. = $_ret, $! = 0;
    $_len;
 }
 
 sub READLINE {
-   my $_id  = $_[0]->[0];
    local $\ = undef if (defined $\);
 
    $_dat_ex->();
    print {$_DAT_W_SOCK} 'O~RLN'.$LF . $_chn.$LF;
-   print {$_DAU_W_SOCK} $_id.$LF . length($/).$LF . $/;
+   print {$_DAU_W_SOCK} $_[0]->[0].$LF . length($/).$LF . $/;
 
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF); my $_buf;
    chomp(my $_ret = <$_DAU_W_SOCK>);
    chomp(my $_len = <$_DAU_W_SOCK>);
 
-   read($_DAU_W_SOCK, my ($_buf), $_len) if $_len;
-   $_dat_un->();
+   if ($_len) {
+      if ($_len < 0) {
+         $_dat_un->(); $. = 0, $! = $_len * -1;
+         return undef;
+      }
+      read($_DAU_W_SOCK, $_buf, $_len);
+   }
 
-   $. = $_ret;
+   $_dat_un->(); $. = $_ret, $! = 0;
    $_buf;
 }
 
@@ -2156,7 +2120,7 @@ sub PRINTF {
 sub WRITE {
    my $_id  = shift()->[0];
    local $\ = undef if (defined $\);
-   local $/ = $LF if (!$/ || $/ ne $LF);
+   local $/ = $LF if ($/ ne $LF);
 
    if (@_ == 1 || (@_ == 2 && $_[1] == length($_[0]))) {
       $_dat_ex->();
@@ -2250,8 +2214,8 @@ sub pending {
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## Methods optimized for
-##  PDL, MCE::Shared::{ Array, Hash, Ordhash, Scalar }, and Hash::Ordered.
+## Methods optimized for:
+##  MCE::Shared::{ Array, Hash, Ordhash, Scalar } and similar.
 ##
 ###############################################################################
 
@@ -2345,7 +2309,7 @@ MCE::Shared::Server - Server/Object packages for MCE::Shared
 
 =head1 VERSION
 
-This document describes MCE::Shared::Server version 1.006
+This document describes MCE::Shared::Server version 1.007
 
 =head1 DESCRIPTION
 
